@@ -1,10 +1,10 @@
 ;;; nethack-api.el -- Emacs interface the lisp window-port
 
-;; Copyright (C) 2002  Ryan Yeske and Shawn Betts
+;; Copyright (C) 2002,2003,2005  Ryan Yeske and Shawn Betts
 
 ;; Author: Ryan Yeske
 ;; Created: Sat Mar 18 11:24:02 2000
-;; Version: $Id: nethack-api.el,v 1.87 2003/08/29 20:03:58 sabetts Exp $
+;; Version: $Id: nethack-api.el,v 1.93 2004/11/19 23:05:39 sabetts Exp $
 ;; Keywords: games
 
 ;; This file is free software; you can redistribute it and/or modify
@@ -350,7 +350,12 @@ FIXME: doesnt actually use ATTR!"
 	  (setq key (nh-read-char (concat 
 					(format "(bad %d) " key)
 					ques " ")))))
-    (nh-send (if (= 13 key) default key))))
+    ;; 13, 27, and 7 are abort keys
+    (nh-send (if (or (= 13 key)
+		     (= 27 key)
+		     (= 7 key))
+		 default
+	       key))))
 
 (defun nhapi-ask-direction (prompt)
   "Prompt the user for a direction"
@@ -509,7 +514,7 @@ The TYPE argument is legacy and serves no real purpose."
       (erase-buffer)
       (if nethack-use-tiles
 	  (progn ;; FIXME: test to see if emacs is capable of tiles
-	    (require 'nethack-glyphs)
+	    (require 'nethack-tiles)
 	    ;; initialize the map with empty tiles
 	    (dotimes (i nh-map-height)
 	      (dotimes (j nh-map-width)
@@ -604,18 +609,18 @@ was actually toggled."
       (save-excursion
 	(let ((inhibit-read-only t))
 	  (goto-char (point-min))
-	  (while (re-search-forward "^[a-zA-Z*] \\([-+]\\|[0-9]+\\) .+$" nil t)
-	    (let ((value (match-string 1)))
+	  (while (re-search-forward nh-menu-item-regexp nil t)
+	    (let ((value (match-string 2)))
 	      (if (string-equal value "-")
-		  (replace-match "+" nil nil nil 1)
-		(replace-match "-" nil nil nil 1))))))))
+		  (replace-match "+" nil nil nil 2)
+		(replace-match "-" nil nil nil 2))))))))
 
 (defun nh-menu-goto-next ()
   "Move to the next selectable menu item."
   (interactive)
   (let ((old-point (point)))
     (goto-char (line-end-position))
-    (goto-char (if (re-search-forward "^[a-zA-Z*] [-+]\\|[0-9]+ .+$" nil t)
+    (goto-char (if (re-search-forward nh-menu-item-regexp nil t)
 		   (line-beginning-position)
 		 old-point))))
 
@@ -624,7 +629,7 @@ was actually toggled."
   (interactive)
   (let ((old-point (point)))
     (goto-char (line-beginning-position))
-    (goto-char (if (re-search-backward "^[a-zA-Z*] [-+]\\|[0-9]+ .+$" nil t)
+    (goto-char (if (re-search-backward nh-menu-item-regexp nil t)
 		   (line-beginning-position)
 		 old-point))))
 
@@ -637,7 +642,7 @@ displayed."
   (interactive)
   (goto-char (point-min))
   (let ((menu-data nil))
-    (while (re-search-forward "^\\([a-zA-Z*]\\) \\([-+]\\|[0-9]+\\) .+$" nil t)
+    (while (re-search-forward nh-menu-item-regexp nil t)
       (let ((accelerator (string-to-char (match-string 1)))
 	    (value (match-string 2)))
 	(cond ((string-equal value "+")
@@ -658,7 +663,7 @@ displayed."
   (let ((inhibit-read-only t))
     (goto-char (point-min))
     ;; turn off all the options
-    (while (re-search-forward "^[a-zA-Z*] \\([-+]\\|[0-9]+\\) .+$" nil t)
+    (while (re-search-forward nh-menu-item-regexp nil t)
       (replace-match "-" nil nil nil 1)))
   (nh-menu-submit))
 
@@ -700,7 +705,8 @@ buffer."
 			(if preselected ?+ ?-)
 			str)))
       (put-text-property start (point) 'face (nh-attr-face attr))
-      (insert-char ?\n 1 nil))))
+      (insert-char ?\n 1 nil)
+      (run-hooks 'nethack-add-menu-hook))))
 
 ;; FIXME: xemacs propertize bug here
 (defun nhapi-end-menu (window prompt)
@@ -776,7 +782,50 @@ the menu is dismissed."
     (insert (concat who " -- " message) "\n")))
 
 (defun nhapi-end ()
-  (message "Goodbye."))
+  (message "Goodbye.")
+  (run-hooks 'nethack-end-hook))
+
+;; Options
+(defvar nh-options-cbreak nil)
+(defvar nh-options-dec-graphics nil)
+(defvar nh-options-echo nil)
+(defvar nh-options-ibm-graphics nil)
+(defvar nh-options-msg-history nil)
+(defvar nh-options-num-pad nil)
+(defvar nh-options-news nil)
+(defvar nh-options-window-inited nil)
+(defvar nh-options-vision-inited nil)
+(defvar nh-options-menu-tab-sep nil)
+(defvar nh-options-menu-requested nil)
+(defvar nh-options-num-pad-mode nil)
+(defvar nh-options-purge-monsters nil)
+(defvar nh-options-bouldersym nil)
+(defvar nh-options-travelcc nil)
+(defvar nh-options-sanity-check nil)
+(defvar nh-options-mon-polycontrol nil)
+
+(defun nhapi-options (cbreak dec-graphics echo ibm-graphics msg-history
+			     num-pad news window-inited vision-inited
+			     menu-tab-sep menu-requested num-pad-mode
+			     purge-monsters bouldersym travelcc
+			     sanity-check mon-polycontrol &rest ignore)
+  (setq nh-options-cbreak cbreak)
+  (setq nh-options-dec-graphics dec-graphics)
+  (setq nh-options-echo echo)
+  (setq nh-options-ibm-graphics ibm-graphics)
+  (setq nh-options-msg-history msg-history)
+  (setq nh-options-num-pad num-pad)
+  (setq nh-options-news news)
+  (setq nh-options-window-inited window-inited)
+  (setq nh-options-vision-inited vision-inited)
+  (setq nh-options-menu-tab-sep menu-tab-sep)
+  (setq nh-options-menu-requested menu-requested)
+  (setq nh-options-num-pad-mode num-pad-mode)
+  (setq nh-options-purge-monsters purge-monsters)
+  (setq nh-options-bouldersym bouldersym)
+  (setq nh-options-travelcc travelcc)
+  (setq nh-options-sanity-check sanity-check)
+  (setq nh-options-mon-polycontrol mon-polycontrol))
 
 (provide 'nethack-api)
 
